// put your Protection token key here (you can generate it here: https://app.skro.eu/settings/landing-protection/php
const PROTECTION_TOKEN = '';

// How long signature should be valid.
const KEY_TTL = 2 * 60 * 1000; // 2 minutes 
      
// GET parameter with  Skro landing key
const KEY_GET_PARAM = 'key'; 

// redirect users that try to access this LP directly to this URL (ie. you can put smartlink URL here)
const REDIRECT_URL        = '';  // ie. https://www.google.com/

// if the REDIRECT_URL is empty, show this safe page HTML to users that try to access this LP directly
const SAFE_PAGE_HTML      = '<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8"><meta name="viewport" content="width=device-width, initial-scale=1.0"><title>Thank You</title></head><body><h1>Thank You!</h1><p>Your submission has been received.</p></body></html>'; 

// in case you don't put an URL in REDIRECT_URL and you don't put a HTML under SAFE_PAGE_HTML the visitor will see a 'Access denied.' message if error occurs.





/* ------------ DO NOT MODIFY BELOW THIS LINE --------------------- */

export default {
  async fetch(request, env) {
    const url = new URL(request.url);
    const key = url.searchParams.get(KEY_GET_PARAM);
    const pathname = url.pathname;
    const filename = pathname.split('/').pop();


    // Request for a non HTML file? (images, scripts)
    if (filename && !filename.endsWith('.htm') && !filename.endsWith('.html')) {
        return env.ASSETS.fetch(request);
    }


    // PROTECTION_TOKEN empty?
    if( !PROTECTION_TOKEN ){
        return new Response('PROTECTION_TOKEN not defined.', { status: 403 });
    }


    // SIGNATURE_TTL empty?
    if( !KEY_TTL ){
        return new Response('KEY_TTL not defined.', { status: 403 });
    }

    // GET param empty
    if (!key) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }

    // decode key
    let decodedKey;
    try {
        decodedKey = atob(decodeURIComponent(key));
        decodedKey = JSON.parse(decodedKey);
    } catch (error) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }

    // extract timestamp and hash
    const { timestamp, hash } = decodedKey;
    if (!timestamp || !hash) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }

    // generate signedHash 
    const signedHash = await generateHMAC(timestamp, PROTECTION_TOKEN);
    
    // compare hash values
    if ( signedHash !== hash ) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }

    // compare TTL
    const currentTime = Date.now();
    const timeDifference = (timestamp * 1000 + KEY_TTL) - currentTime;
    if( timeDifference < 0 ){
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }

    // Access granted, fetch from env.ASSETS
    return env.ASSETS.fetch(request);

    
  }
};


async function generateHMAC(timestamp, secretKey) {
  const secretKeyArrayBuffer = new TextEncoder().encode(secretKey);
  const timestampArrayBuffer = new TextEncoder().encode(timestamp.toString());
  const importedKey = await crypto.subtle.importKey(
    'raw',
    secretKeyArrayBuffer,
    { name: 'HMAC', hash: { name: 'SHA-1' } },
    false,
    ['sign']
  );
  const signatureArrayBuffer = await crypto.subtle.sign(
    'HMAC',
    importedKey,
    timestampArrayBuffer
  );

  return Array.prototype.map.call(
    new Uint8Array(signatureArrayBuffer),
    x => ('00' + x.toString(16)).slice(-2)
  ).join('');
}


/* ------------ DO NOT MODIFY ABOVE THIS LINE --------------------- */